	using UnityEngine;

	public static class Perlin
	{
		static int[] p;
		static int[] permutation = {
			151, 160, 137, 91, 90, 15, 131, 13, 201, 95, 96, 53, 194, 233, 7, 225,
			140,36,103,30,69,142,8,99,37,240,21,10,23,190,6,148,
			247,120,234,75,0,26,197,62,94,252,219,203,117,35,11,32,
			57,177,33,88,237,149,56,87,174,20,125,136,171,168, 68,175,
			74,165,71,134,139,48,27,166,77,146,158,231,83,111,229,122,
			60,211,133,230,220,105,92,41,55,46,245,40,244,102,143,54,
			65,25,63,161, 1,216,80,73,209,76,132,187,208, 89,18,169,
			200,196,135,130,116,188,159,86,164,100,109,198,173,186, 3,64,
			52,217,226,250,124,123,5,202,38,147,118,126,255,82,85,212,
			207,206,59,227,47,16,58,17,182,189,28,42,223,183,170,213,
			119,248,152, 2,44,154,163, 70,221,153,101,155,167, 43,172,9,
			129,22,39,253, 19,98,108,110,79,113,224,232,178,185, 112,104,
			218,246,97,228,251,34,242,193,238,210,144,12,191,179,162,241,
			81,51,145,235,249,14,239,107,49,192,214, 31,181,199,106,157,
			184, 84,204,176,115,121,50,45,127, 4,150,254,138,236,205,93,
			222,114,67,29,24,72,243,141,128,195,78,66,215,61,156,180
		};
	
		static Perlin()
		{
			p = new int[512];
			for (int i=0; i < 256 ; i++)
			{
				p[256 + i] = p[i] = permutation[i]; 
			}
		}
	
		public static Vector3 NoiseVector3(Vector3 position) 
		{
		   return new Vector3(
			Noise(position.x, position.y, position.z),
			Noise(position.y, position.z, position.x),
			Noise(position.z, position.x, position.y)
			)
		   ;
		   //return Noise(position.x + 0.5f, position.y + 0.5f, position.z + 0.5f);
		}

        public static float Noise(Vector3 position) 
		{
		   return Noise(position.x, position.y, position.z);
		   //return Noise(position.x + 0.5f, position.y + 0.5f, position.z + 0.5f);
		}

        public static float Noise(float x)
		{
            var floorX = floor(x);
            //var floorX = Mathf.Floor(x);
            int X = ((int)floorX) & 255;                  // FIND UNIT CUBE THAT
			x -= floorX;                                // FIND RELATIVE X,Y,Z
			float u = fade(x);                                // COMPUTE FADE CURVES
			float hx = x;// * 2.0f - 1.0f;
			
			int A = p[X];
			int AA = p[A];
			int B = p[X+1];
			int BA = p[B];
		   
			return lerp(u, grad(p[AA], hx), grad(p[BA], hx - 1.0f)); // BLENDED
		}

        public static float Noise(float x, float y, float z) 
		{
            var floorX = floor(x);
            int X = ((int)floorX) & 255;                  // FIND UNIT CUBE THAT
            x -= floorX;                                // FIND RELATIVE X,Y,Z
            var floorY = floor(y);
            int Y = ((int)floorY) & 255;                  // CONTAINS POINT.
            y -= floorY;                                // OF POINT IN CUBE.
            var floorZ = floor(z);
			int Z = ((int)floorZ) & 255;
            z -= floorZ;
			float u = fade(x);                                // COMPUTE FADE CURVES
			float v = fade(y);                                // FOR EACH OF X,Y,Z.
			float w = fade(z);
			float hx = x;// * 2.0f - 1.0f;
			float hy = y;// * 2.0f - 1.0f;
			float hz = z;// * 2.0f - 1.0f;
			
			int A = p[X]+Y;
			int AA = p[A]+Z;
			int AB = p[A+1]+Z;      // HASH COORDINATES OF
			int B = p[X+1]+Y;
			int BA = p[B]+Z;
			int BB = p[B+1]+Z;      // THE 8 CUBE CORNERS,
	
			//~ if(Y > -2 && Y < 2 && X > -4 && X < 4 && Z > -4 && Z < 4)
			//~ {
				//~ Debug.Log(p[AA] + "," + p[BA] + "," + x + "," + y + "," + z + "=" + lerp(u, grad(p[AA], x  , y  , z   ), grad(p[BA], x - 1.0f, y  , z   )));
				//~ Debug.Log(p[AB] + "," + p[BB] + "," + x + "," + y + "," + z + "=" + lerp(u, grad(p[AA], x  , y  , z   ), grad(p[BA], x - 1.0f, y  , z   )));
			//~ }
	
			return lerp(w,
				lerp(v,
					lerp(u, grad(p[AA], hx  , hy  , hz   ), grad(p[BA], hx - 1.0f, hy  , hz   )), // BLENDED
	                lerp(u, grad(p[AB], hx  , hy - 1.0f, hz   ), grad(p[BB], hx - 1.0f, hy - 1.0f, hz   ))),// FROM  8
	            lerp(v, 
					lerp(u, grad(p[AA+1], hx  , hy  , hz - 1.0f ), grad(p[BA+1], hx - 1.0f, hy  , hz - 1.0f )), // OF CUBE
	                lerp(u, grad(p[AB+1], hx  , hy - 1.0f, hz - 1.0f ), grad(p[BB+1], hx - 1.0f, hy - 1.0f, hz - 1.0f ))));
		}

        private static int floor(float x)
        {
            var xi = (int)x;
            return x < xi ? xi - 1 : xi;
        }
        
        private static float fade(float t)
		{
			return t * t * t * (t * (t * 6 - 15) + 10); 
		}

        private static float lerp(float t, float a, float b)
		{
			return a + t * (b - a); 
		}

        private static float grad(int hash, float x)
		{
			int h = hash & 1;                      // CONVERT LO 4 BITS OF HASH CODE
			return(h == 1 ? x : -x);
		}

        private static float grad(int hash, float x, float y, float z)
		{
			int h = hash & 15;                      // CONVERT LO 4 BITS OF HASH CODE
			float u = (h < 8) ? x : y;                 // INTO 12 GRADIENT DIRECTIONS.
			float v = (h < 4) ? y : ((h == 12 || h == 14) ? x : z);
			return (((h & 1) == 0) ? u : -u) + (((h & 2) == 0) ? v : -v);
		}
	}
